@@ -11,109 +11,7 @@ stopif = function(cond,...){
 }
 
 
-######## build_criterion ###########
-build_criterion = function(criterion,dfs){
-    # dfs should be data.frame
-    # build criterion should return logical matrix with the form of dfs (e. g. the same dimension)
-    UseMethod("build_criterion")
-}
 
-#' @export
-build_criterion.function = function(criterion,dfs){
-    res = lapply(dfs,function(colmn){
-        cond = criterion(colmn)
-        stopif(length(cond)!=length(colmn),"Cells number of criterion doesn't equal cells number of argument.")
-        if_na(as.logical(cond), FALSE)
-    })
-    do.call(cbind,res)
-}
-
-#' @export
-build_criterion.default = function(criterion,dfs){
-    res = lapply(dfs, function(colmn){
-        if(("POSIXct" %in% class(colmn)) & !("POSIXct" %in% class(criterion))){
-            criterion = as.POSIXct(criterion)
-        }
-        colmn %in% criterion
-    })
-    do.call(cbind,res)
-}
-
-
-#' @export
-build_criterion.logical = function(criterion,dfs){
-    # uncertainty if criterion is result of something is.na(dfs[,i]) 
-    # should we count NA in such case - possible solution - forbid logical criterion for count if
-    if (is.atomic(criterion) && (length(criterion)==1) && is.na(criterion)) {
-        return(build_criterion(as.numeric(criterion), dfs))
-    }
-    check_conformance(dfs, criterion)
-    res = matrix(nrow = NROW(dfs), ncol = NCOL(dfs))
-    if(NCOL(criterion)>1){
-        for(i in seq_along(dfs)){
-            res[,i] = criterion[,i] 
-        }
-    } else {
-        for(i in seq_along(dfs)){
-            res[,i] = criterion 
-        }
-    }
-    if_na(res, FALSE)
-}
-
-#' @export
-build_criterion.data.frame = function(criterion,dfs){
-    build_criterion(as.matrix(criterion), dfs)
-}
-
-#' @export
-build_criterion.matrix = function(criterion,dfs){
-    stopif(!is.logical(criterion), "matrix/data.frame criterion should be logical.")
-    build_criterion.logical(criterion, dfs)
-}
-
-#' @export
-build_criterion.list = function(criterion,dfs){
-    stop("Condition of type 'list' doesn't supported.")
-    #stopif(length(criterion)==0, "Zero-length list is provided as argument.")
-    #res = lapply(seq_along(criterion), function(i) build_criterion(column(criterion, i), as.data.frame(column(dfs, i))))
-    #do.call(cbind, res)
-}
-
-#' @export
-build_criterion.criterion = function(criterion,dfs){
-    build_criterion.function(criterion,dfs) 
-}
-
-
-###### check_conformance ################
-
-# value should be ncol(value)==1 or ncol(value) = ncol(x) 
-# value should be nrow(value)==1 or nrow(value) = nrow(x) 
-check_conformance = function(x,value){
-    UseMethod("check_conformance")
-}
-
-#' @export
-check_conformance.default = function(x,value){
-    stopif(length(value)==0, "'value' has zero length.")
-    if(is.list(value) && !is.data.frame(value)){
-        stopif(length(value)>1 && NCOL(x)!=length(value), "Length of 'value' should be 
-               1 or equals to number of columns of 'x' but length(value)=",length(value),", NCOL(x)=", NCOL(x))
-    } else {
-        stopif(NCOL(value)>1 && NCOL(x)!=NCOL(value), "Number of columns in 'value' should be 
-               1 or equals to number of columns of 'x' but NCOL(value)=",NCOL(value),", NCOL(x)=", NCOL(x))
-        stopif(NROW(value)>1 && NROW(x)!=NROW(value), "Number of rows in 'value' should be
-               1 or equals number of rows of 'x' but NROW(value)=",NROW(value),", NROW(x)=", NROW(x))
-    }
-    invisible(TRUE)
-}
-
-#' @export
-check_conformance.list = function(x, value){
-    
-    invisible(TRUE)    
-}
 
 ####### column ###########
 
@@ -176,145 +74,12 @@ column.default = function(x, column_num, condition = NULL){
     
 }    
 
-#######
-"column<-" = function(x, column_num, condition = NULL, value){
-    UseMethod("column<-")
-}
-
-#' @export
-"column<-.data.frame" = function(x, column_num, condition = NULL, value){
-    stopif(column_num>ncol(x), "Too large column_num:",column_num, " only ", ncol(x), " columns in the data.frame.")
-    column(x[[column_num]], 1, condition = condition) = value
-    x
-}
-
-#' @export
-"column<-.matrix" = function(x, column_num, condition = NULL, value){
-    stopif(column_num>ncol(x), "Too large column_num:",column_num, " only ", ncol(x), " columns in the matrix.")
-    column(x[, column_num], 1, condition = condition) = value
-    x
-}
-
-#' @export
-"column<-.list" = function(x, column_num, condition = NULL, value){
-    stop("Assignment for list doesn't implemented.")
-    
-}
-
-#' @export
-"column<-.default" = function(x, column_num, condition = NULL, value){
-    if(is.factor(value)){
-        x = as.factor(x)
-        column(x, column_num = column_num, condition) = value
-        return(x)
-    } 
-    
-    if(inherits(value, "POSIXct") && (is.logical(x) || is.numeric(x))){
-        # first assignment - we expect that x with all NA and set its class to POSIXct
-        x = as.POSIXct(x)
-        column(x, column_num = column_num, condition) = value
-        return(x)
-    } 
-    if(is.null(condition)){
-        x[] = value
-    } else {
-        x[condition] = value
-    } 
-    
-    x
-}  
-
-#' @export
-"column<-.factor" = function(x, column_num, condition = NULL, value){
-    fac_levels = levels(x)
-    if(!all(value %in% fac_levels)){
-        fac_levels = union(fac_levels, value)
-        levels(x) = fac_levels
-    }
-    if(is.null(condition)){
-        x[] = value
-    } else {
-        x[condition] = value
-    }     
-    x
-}  
-
-###########################
-# use this function only inside other functions
-# eval_dynamic_scoping = function(expr, envir, skip_up_to_frame = ""){
-#     all_env = rev(sys.frames())[-(1:2)] # remove current and calling environement
-#     sys_calls = lapply(rev(sys.calls())[-(1:2)], function(each_call){
-#         res = as.character(as.list(each_call)[[1]])
-#         if(res[1] %in% c("::", ":::")){
-#             res[3]
-#         } else {
-#             res[1]
-#         }
-#     })
-#     sys_calls = unlist(sys_calls)
-#     skip = stats::na.omit(match(skip_up_to_frame, sys_calls))
-#     if(length(skip)==0) {
-#         skip = 0
-#     } else {
-#         skip = max(skip)
-#     }    
-#     
-#     if(skip>0){
-#         all_env = c(all_env[-seq_len(skip)], .GlobalEnv) 
-#     } else {
-#         all_env = c(all_env, .GlobalEnv) 
-#     }
-#     
-#     succ = FALSE
-#     i = 1
-#     while(!succ && i<=length(all_env)){
-#         succ = TRUE
-#         parent.env(envir) = all_env[[i]]
-#         res = tryCatch(eval(expr, envir), error = function(e) {succ<<-FALSE})
-#         if(!succ) i = i + 1
-#     }
-#     stopif(!succ, "`", deparse(substitute(expr)),"` - some variables not found.")
-#     res
-# }
-
-
-
-#############################
-
-#########################################
-
-valid = function(x){
-    UseMethod("valid")
-}
-
-#' @export
-valid.default = function(x){
-    !is.na(x)
-}
-
-#' @export
-valid.data.frame = function(x){
-    if (length(x)) {
-        res = do.call(cbind, lapply(x, is.na))
-    } else {
-        res = matrix(FALSE, NROW(x), 0)
-    }    
-    !rowAlls(res)
-}
-
-#' @export
-valid.dichotomy = function(x){
-    rowSums(x, na.rm = TRUE)>0
-}
-
-#' @export
-valid.matrix = function(x){
-    !rowAlls(is.na(x))
-}
-
 
 
 
+ENV_INTERNAL_NAMES = c(".n", ".N", "..", 
+                       ".new_var", ".new_character", 
+                       ".new_numeric", ".new_logical")
 ###########
 
 prepare_env = function(env, n, column_names){
@@ -329,19 +94,11 @@ prepare_env = function(env, n, column_names){
         env$.internal_column_names0 = column_names
         lockBinding(".internal_column_names0", env)
     }
-    lockBinding(".n", env)
-    lockBinding(".N", env)
-    lockBinding(".new_var", env)
-    lockBinding(".new_character", env)
-    lockBinding(".new_numeric", env)
-    lockBinding(".new_logical", env)
-
+    lapply(ENV_INTERNAL_NAMES %d% "..", lockBinding, env)
 }
 
 clear_env = function(env){
-    rm(".n", ".N", "..", 
-       ".new_var", ".new_character", 
-       ".new_numeric", ".new_logical",
+    rm(list = ENV_INTERNAL_NAMES,
        envir = env)  
     if(exists(".internal_column_names0", envir = env)) rm(".internal_column_names0", envir = env)
 }
@@ -352,8 +109,7 @@ get_current_variables = function(envir){
         if(exists(".internal_column_names0", envir =envir)){
             column_names = envir[[".internal_column_names0"]]
             curr = ls(envir = envir, all.names = TRUE, sorted = FALSE)
-            curr = curr %d% c(".n",  ".N", ".internal_column_names0", 
-                              "..", ".new_var", ".new_character", ".new_numeric", ".new_logical")
+            curr = curr %d% c(ENV_INTERNAL_NAMES, ".internal_column_names0")
             # removed = names(curr)[vapply(curr, is.null, NA, USE.NAMES = FALSE)]
             # curr = names(curr) %d% removed # remove deleted variables?
             new_names = column_names %i% curr 
@@ -376,9 +132,7 @@ new_var_generator = function(FUN, number_of_rows){
 
 
 ### TRUE if argument is list, not data.frame
-is_list=function(x)
-    
-{
+is_list=function(x){
     is.list(x) && (!is.data.frame(x))
 }
 
@@ -435,8 +189,7 @@ integer_encoding.data.frame=function(x, dict = NULL){
 ### list(a,list(b,c))->list(a,b,c)
 ### flat_df = FALSE data.frame will be left as data.frame
 ### flat_df = TRUE data.frame will be converted to list
-flat_list=function(x, flat_df = FALSE)
-{
+flat_list=function(x, flat_df = FALSE){
     if(flat_df){
         check_list = is.list
     } else {
@@ -673,6 +426,24 @@ fast_match = function(x, table, nomatch = NA_integer_, NA_incomparable = FALSE){
     ind
 }
 
+fast_in = function(x, value){
+    if(is.numeric(value) && 
+       length(value)>0 && 
+       length(value)<3 && 
+       !anyNA(value) && 
+       !any(is.infinite(value)) && 
+       is.numeric(x)){
+        # optimization for very special and very frequent case after profiling
+        res = x == value[[1]]
+        for(each in value[-1]){
+            res = res | (x==each)
+        }
+        res & !is.na(res)
+    } else {
+        fast_match(x, value, nomatch = 0L)>0
+    }
+}
+
 #################
 
 add_class = function(x, ...){
@@ -732,4 +503,13 @@ get_named_expressions = function(dots){
         vnames[is.na(vnames)] = ""
     }
     setNames(res, vnames)
+}
+
+############
+
+# for assignment functions
+# to avoid warning about shallow copy
+fix_datatable = function(x){
+    if(is.data.table(x)) setDT(x, check.names = FALSE)
+    x
 }
\ No newline at end of file
