@@ -57,13 +57,24 @@ add_rows = function(..., nomatch_columns = c("add", "drop", "stop")){
 #' @rdname add_rows
 #' @export
 add_rows.data.frame = function(..., nomatch_columns = c("add", "drop", "stop")){
+    args = list(...)
+    is_dt = unlist(lapply(args, is.data.table), use.names = FALSE)
+    if(any(is_dt)){
+        # not optimal solution for data.table 
+        # TODO all should be rewritten with rbindlist
+        args = lapply(list(...), as.sheet)        
+    }
+    
     f = function(x, y){
         force(nomatch_columns)
         add_rows1(x, y, nomatch_columns = nomatch_columns)
     }
-    Reduce(f, list(...))
+    Reduce(f, args)
 }
 
+
+
+
 #' @export
 add_rows.default =  function(..., nomatch_columns = c("add", "drop", "stop")){
     rbind(...)
@@ -137,7 +148,7 @@ add_rows1 = function(x, y, nomatch_columns = c("add", "drop", "stop")){
             stopif(nomatch_columns == "stop", "Different column names in 'x' and 'y'.")
         }
         res = rbind.data.frame(x, y, stringsAsFactors = FALSE)
-        if_val(colnames(res), from = temp_names) = true_names
+        recode(colnames(res)) = from_to(from = temp_names, to = true_names)
         res
     } else {
         rbind.data.frame(x, y, stringsAsFactors = FALSE)
